require 'mirror-mirror/mirror_visitor'

module MirrorMirror
  # Sass should provide hooks for getting access to the AST a various stages of transformation.
  # So that we don't have to monkeypatch it.
  module SassPatch
    def self.included(base)
      if base.respond_to?(:css_tree, true)
        # Sass >= 3.3
        base.send(:alias_method, :css_tree_without_mirror_mirror, :css_tree)
        base.send(:alias_method, :css_tree, :css_tree_with_mirror_mirror)
      else
        # Sass < 3.3
        base.send(:alias_method, :render_without_mirror_mirror, :render)
        base.send(:alias_method, :render, :render_with_mirror_mirror)
      end
    end

    def css_tree_with_mirror_mirror
      root = css_tree_without_mirror_mirror
      MirrorVisitor.visit(root)
      root
    end

    def render_with_mirror_mirror
      Sass::Tree::Visitors::CheckNesting.visit(self)
      result = Sass::Tree::Visitors::Perform.visit(self)
      Sass::Tree::Visitors::CheckNesting.visit(result) # Check again to validate mixins
      result, extends = Sass::Tree::Visitors::Cssize.visit(result)
      Sass::Tree::Visitors::Extend.visit(result, extends)
      MirrorVisitor.visit(result)
      result.to_s
    end
  end

  module CssizePatch
    def visit_directive(directive)
      if directive.resolved_value.start_with?("@-mirror-mirror")
        did_bubble = bubble(directive)
        yield unless did_bubble
        directive
      else
        yield
        directive
      end
    end
  end

  module DirectiveNodePatch
    def self.included(base)
      base.send(:attr_accessor, :group_end)
    end

    def bubbles?
      if resolved_value
        resolved_value.start_with?("@-mirror-mirror")
      else
        value.first.start_with?("@-mirror-mirror")
      end
    end
  end
end
