require 'set'
module MirrorMirror::Transformation
  class LinearGradient < PropertyTransformation
    IMAGE_PROPERTIES = Set[
      'background',
      'background-image',
      'border-image',
      'list-style-image',
      'list-style',
      'content'
    ]
    def transform_expression?(name)
      IMAGE_PROPERTIES.include?(name)
    end

    def transform_expression(name, expression)
      for_each_value(expression) do |e|
        if grad = find_linear_gradient(e)
          if direction?(grad.args[0])
            grad.args[0] = flip(grad.args[0])
          end
          e
        else
          e
        end
      end
    end

    def find_linear_gradient(expression)
      case expression
      when Sass::Script::List
        expression.value.each do |v|
          grad = find_linear_gradient(v)
          return grad if grad
        end
        nil
      when Sass::Script::Funcall
        expression if expression.name =~ /^(-(moz|webkit|o|ms)-)?(repeating-)?linear-gradient$/
      end
    end

    def degree?(e)
      e.is_a?(Sass::Script::Number) && e.numerator_units.first == "deg"
    end

    DIRECTIONAL_KEYWORDS = Set.new(%w(to top left right bottom))
    def directional_keywords?(e)
      (e.is_a?(Sass::Script::List) && e.value.first.is_a?(Sass::Script::String) && DIRECTIONAL_KEYWORDS.include?(e.value.first.value)) ||
        e.is_a?(Sass::Script::String) && DIRECTIONAL_KEYWORDS.include?(e.value)
    end

    def direction?(e)
      degree?(e) || directional_keywords?(e)
    end

    def color_stop?(e)
      e.is_a?(Sass::Script::Color) ||
        (e.is_a?(Sass::Script::List) && e.value.first.is_a?(Sass::Script::Color))
    end

    S360 = Sass::Script::Number.new(360)
    def flip(direction)
      if degree?(direction)
        opts(direction.options) { S360.minus(direction).mod(S360) }
      else
        case direction
        when Sass::Script::List
          direction.value.map!{|v| flip(v) }
          direction
        when Sass::Script::String
          if direction.value == "left"
            opts(direction.options) { Sass::Script::String.new("right") }
          elsif direction.value == "right"
            opts(direction.options) { Sass::Script::String.new("left") }
          else
            direction
          end
        else
          direction
        end
      end
    end
  end
end
