module MirrorMirror::Transformation
  class PropertyTransformation < Base
    # The singleton instances, one for each transformation class
    def self.instances
      super.
        select {|i| i.is_a?(PropertyTransformation) && i.class != PropertyTransformation}
    end

    # @param name [String] The property name
    # @return [Boolean]
    def transform_name?(name)
      false
    end

    # @param name [String] The property name
    def transform_name(name)
      name
    end

    # @param name [String] The property name
    # @return [Boolean]
    def transform_value?(name)
      false
    end

    # @param name [String] The property name
    # @return [Boolean]
    def transform_expression?(name)
      false
    end

    # @param property [String] The property whose value is being transformed.
    #   Any prefix/hacks should be removed from the property name.
    # @param value [String] The value being transformed.
    # @return [String] A new value for the property
    def transform_value(property, value)
      value
    end

    # It is safe for this method to mutate the expression that is passed to it.
    #
    # @param property [String] The property whose value is being transformed.
    #   Any prefix/hacks should be removed from the property name.
    # @param expression [Sass::Script::Node] The value being transformed.
    # @return [Sass::Script::Node] A new expression for the property
    def transform_expression(property, expression)
      return expression
    end

  protected

    def opts(options)
      node = yield
      node.options = options
      node
    end

    def for_each_value(expression)
        if expression.is_a?(Sass::Script::List) && expression.separator == :comma
          expression.value.map! {|e| yield(e) }
          expression
        else
          yield(expression)
        end
    end
  end
end


